<?php
// vim: set expandtab tabstop=4 shiftwidth=4 fdm=marker:
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Martin Jansen <mj@php.net>                                  |
// |                                                                      |
// +----------------------------------------------------------------------+
//
// $Id: RSS.php,v 1.1 2006/05/04 18:08:42 michael.schneemann Exp $
//

/**
 * RSS parser class.
 *
 * This class is a parser for Resource Description Framework (RDF) Site
 * Summary (RSS) documents. For more information on RSS see the
 * website of the RSS working group (http://www.purl.org/rss/).
 *
 * @author Martin Jansen <mj@php.net>
 * @version $Revision: 14628 $
 * @access  public
 */
class we_xml_rss{
	// {{{ properties
	/**
	 * @var string
	 */
	var $insideTag = '';

	/**
	 * @var array
	 */
	var $insideTagStack = [];

	/**
	 * @var string
	 */
	var $activeTag = '';

	/**
	 * @var array
	 */
	var $channel = [];
	public $parseError = '';

	/**
	 * @var array
	 */
	var $items = [];
	private $fields = [
		'item' => [],
		'textinput' => [],
		'image' => [],
	];

	/**
	 * @var array
	 */
	var $textinputs = [];

	/**
	 * @var array
	 */
	var $attribs;
	private $last;

	/**
	 * @var array
	 */
	const PARENT_TAGS = ['CHANNEL', 'ITEM', 'IMAGE', 'TEXTINPUT'];

	/**
	 * @var array
	 */
	const CHANNEL_TAGS = ['TITLE', 'LINK', 'DESCRIPTION', 'IMAGE',
		'ITEMS', 'TEXTINPUT', 'LANGUAGE', 'COPYRIGHT',
		'MANAGINGEDITOR', 'WEBMASTER', 'PUBDATE', 'LASTBUILDDATE',
		'CATEGORY', 'GENERATOR', 'DOCS', 'CLOUD', 'TTL',
		'RATING'];

	/**
	 * @var array
	 */
	const ITEM_TAGS = ['TITLE', 'LINK', 'DESCRIPTION', 'PUBDATE', 'AUTHOR', 'CATEGORY',
		'COMMENTS', 'ENCLOSURE', 'GUID', 'PUBDATE', 'SOURCE',
		'CONTENT:ENCODED'];

	/**
	 * @var array
	 */
	const IMAGE_TAGS = ['TITLE', 'URL', 'LINK', 'WIDTH', 'HEIGHT'];
	const TEXTINPUT_TAGS = ['TITLE', 'DESCRIPTION', 'NAME', 'LINK'];

	/**
	 * List of allowed module tags
	 *
	 * Currently Dublin Core Metadata, blogChannel RSS module, CreativeCommons,
	 * Content and Syndication are supported.
	 *
	 * @var array
	 */
	const MODULE_TAGS = ['DC:TITLE', 'DC:CREATOR', 'DC:SUBJECT', 'DC:DESCRIPTION',
		'DC:PUBLISHER', 'DC:CONTRIBUTOR', 'DC:DATE', 'DC:TYPE',
		'DC:FORMAT', 'DC:IDENTIFIER', 'DC:SOURCE', 'DC:LANGUAGE',
		'DC:RELATION', 'DC:COVERAGE', 'DC:RIGHTS',
		'BLOGCHANNEL:BLOGROLL', 'BLOGCHANNEL:MYSUBSCRIPTIONS',
		'BLOGCHANNEL:MYSUBSCRIPTIONS', 'BLOGCHANNEL:CHANGES',
		'CC:LICENSE', 'CONTENT:ENCODED',
		'SY:UPDATEPERIOD', 'SY:UPDATEFREQUENCY', 'SY:UPDATEBASE',
	];

	// }}}
	// {{{ Constructor

	/**
	 * Constructor
	 *
	 * @access public
	 * @param mixed File pointer, name of the RSS file, or an RSS string.
	 * @param string  Source charset encoding, use null (default) to use
	 *                default encoding (ISO-8859-1)
	 * @param string  Target charset encoding, use null (default) to use
	 *                default encoding (ISO-8859-1)
	 * @throws Exception
	 */
	function __construct($data = '', $tgtenc = 'UTF-8'){
		if($data == ''){
			throw new Exception();
		}

		$parser = xml_parser_create();
		xml_parser_set_option($parser, XML_OPTION_TARGET_ENCODING, (strstr($tgtenc, 'ISO-88') ? 'ISO-8859-1' : $tgtenc));

		xml_set_object($parser, $this);
		xml_set_element_handler($parser, 'startHandler', 'endHandler');
		xml_set_character_data_handler($parser, 'cdataHandler');
		//xml_set_default_handler($parser, 'defaultHandler');


		if(!xml_parse($parser, $data, true)){
			$this->parseError = xml_get_current_line_number($parser) . ': ' . xml_Error_string(xml_get_error_code($parser)) . ' - ' .
				xml_get_current_byte_index($parser) . '(' . substr($data, xml_get_current_byte_index($parser) - 10, 20) . ')';
			throw new Exception();
		}
		xml_parser_free($parser);
	}

	// }}}
	// {{{ startHandler()

	/**
	 * Start element handler for XML parser
	 *
	 * @access private
	 * @param  object XML parser object
	 * @param  string XML element
	 * @param  array  Attributes of XML tag
	 * @return void
	 */
	function startHandler($parser, $element, array $attribs){
		$element = str_replace('RSS:', '', $element);

		if(in_array($element, self::PARENT_TAGS)){
			array_push($this->insideTagStack, $element);
		} else {
			switch($element){
				case 'ENCLOSURE' :
					$this->attribs = $attribs;
					break;

				default:
					$this->activeTag = $element;
			}
		}
	}

	// }}}
	// {{{ endHandler()

	/**
	 * End element handler for XML parser
	 *
	 * If the end of <item>, <channel>, <image> or <textinput>
	 * is reached, this method updates the structure array
	 * $this->struct[] and adds the field "type" to this array,
	 * that defines the type of the current field.
	 *
	 * @access private
	 * @param  object XML parser object
	 * @param  string
	 * @return void
	 */
	function endHandler($parser, $element){
		if(substr($element, 0, 4) === "RSS:"){
			$element = substr($element, 4);
		}

		if($element == end($this->insideTagStack)){
			array_pop($this->insideTagStack);

			//$this->struct[] = array_merge(['type' => strtolower($element)], $this->last);
		}

		switch($element){
			case 'ITEM':
				$this->items[] = $this->fields['item'];
				$this->fields['item'] = [];
				break;
			case 'IMAGE':
				$this->images[] = $this->fields['image'];
				$this->fields['image'] = '';
				break;

			case 'TEXTINPUT':
				$this->textinputs[] = $this->fields['textinput'];
				$this->fields['textinput'] = '';
				break;

			case 'ENCLOSURE':
				if(!isset($this->fields['item']['enclosures'])){
					$this->fields['item']['enclosures'] = [];
				}

				$this->fields['item']['enclosures'][] = array_change_key_case($this->attribs, CASE_LOWER);
				$this->attribs = [];
				break;
		}

		$this->activeTag = '';
	}

	// }}}
	// {{{ cdataHandler()

	/**
	 * Handler for character data
	 *
	 * @access private
	 * @param  object XML parser object
	 * @param  string CDATA
	 * @return void
	 */
	function cdataHandler($parser, $cdata){
		if(($tag = end($this->insideTagStack))){
			$tagName = strtolower($tag);
			$var = constant('self::' . strtoupper($tagName) . '_TAGS');

			if(in_array($this->activeTag, $var) || in_array($this->activeTag, self::MODULE_TAGS)){
				$this->_add($tagName, strtolower($this->activeTag), $cdata);
			}
		}
	}

	// }}}
	// {{{ defaultHandler()

	/**
	 * Default handler for XML parser
	 *
	 * @access private
	 * @param  object XML parser object
	 * @param  string CDATA
	 * @return void
	 */
	function defaultHandler($parser, $cdata){
		return;
	}

	// }}}
	// {{{ _add()

	/**
	 * Add element to internal result sets
	 *
	 * @access private
	 * @param  string Name of the result set
	 * @param  string Fieldname
	 * @param  string Value
	 * @return void
	 * @see    cdataHandler
	 */
	private function _add($type, $field, $value){
		if(empty($this->fields[$type]) || empty($this->fields[$type][$field])){
			$this->fields[$type][$field] = $value;
		} else {
			$this->fields[$type][$field] .= $value;
		}

		$this->last = $this->fields[$type];
	}

	// }}}
	// {{{ getStructure()

	/**
	 * Get complete structure of RSS file
	 *
	 * @access public
	 * @return array
	 */
	function getStructure(){
		return (array) $this->struct;
	}

	// }}}
	// {{{ getchannelInfo()

	/**
	 * Get general information about current channel
	 *
	 * This method returns an array containing the information
	 * that has been extracted from the <channel>-tag while parsing
	 * the RSS file.
	 *
	 * @access public
	 * @return array
	 */
	function getChannelInfo(){
		return (array) $this->channel;
	}

	// }}}
	// {{{ getItems()

	/**
	 * Get items from RSS file
	 *
	 * This method returns an array containing the set of items
	 * that are provided by the RSS file.
	 *
	 * @access public
	 * @return array
	 */
	function getItems(){
		return (array) $this->items;
	}

	// }}}
	// {{{ getImages()

	/**
	 * Get images from RSS file
	 *
	 * This method returns an array containing the set of images
	 * that are provided by the RSS file.
	 *
	 * @access public
	 * @return array
	 */
	function getImages(){
		return (array) $this->images;
	}

	// }}}
	// {{{ getTextinputs()

	/**
	 * Get text input fields from RSS file
	 *
	 * @access public
	 * @return array
	 */
	function getTextinputs(){
		return (array) $this->textinputs;
	}

	// }}}
}
